#include <qlayout.h>
#include <qscrollview.h>
#include <qtooltip.h>
#include <qbuttongroup.h>

#include "MapEditor.h"

/**
 * MapEditor class extends QWidget and acts as a parent widget for QScrollBars, QRadioButtons, QLabels,
 * etc.  It is used as a controller class between a MapViewer object and a CrossSection object.  It uses
 * it's child widgets to trigger events such that a new slice or plane can be calculated and 
 * displayed in the MapViewer widget.
 */
MapEditor::MapEditor(svt_volume<double> *volumericMap, QWidget *parent, const char *name, WFlags flags) : QWidget(parent, name, flags | WNoAutoErase) {
	m_currentVolumericMap = volumericMap;
	m_ordinalVolumericMap = NULL;
	m_currentPlane = 'y';
	m_factorValue = 1;
	m_crossSectionVolumericMap = NULL;
	m_crossSectionVolumericMapMask = NULL;
	m_ordinalVolumericMapMask = (char *) new char[m_currentVolumericMap->getNumberVoxels()];
	
	getCrossSection(m_currentPlane, m_currentVolumericMap->getSizeY()/2);
	for(int i = 0; i < m_currentVolumericMap->getNumberVoxels(); i++ )
		m_ordinalVolumericMapMask[i] = 0;
	initialise();
	
	QObject::connect(m_zoomScrollBar, SIGNAL(valueChanged(int)), 
					m_mapViewer, SLOT(setZoomScale(int)));
	QObject::connect(m_planeScrollBar, SIGNAL(valueChanged(int)), 
					this, SLOT(getNewCrossSection(int)));
	QObject::connect(m_binaryScrollBar, SIGNAL(valueChanged(int)), 
					m_mapViewer, SLOT(setBinaryThreshold(int)));
	QObject::connect(m_gammaScrollBar, SIGNAL(valueChanged(int)), 
					this, SLOT(updateGammaLabel(int)));
	QObject::connect(m_gammaScrollBar, SIGNAL(valueChanged(int)), 
					m_mapViewer, SLOT(setGammaCurve(int)));
	QObject::connect(m_xPlane, SIGNAL(clicked()), 
					this, SLOT(setXPlaneView()));
	QObject::connect(m_yPlane, SIGNAL(clicked()), 
					this, SLOT(setYPlaneView()));
	QObject::connect(m_zPlane, SIGNAL(clicked()), 
					this, SLOT(setZPlaneView()));
	QObject::connect(m_xProjection, SIGNAL(clicked()), 
					this, SLOT(setXProjectionView()));
	QObject::connect(m_yProjection, SIGNAL(clicked()), 
					this, SLOT(setYProjectionView()));
	QObject::connect(m_zProjection, SIGNAL(clicked()), 
					this, SLOT(setZProjectionView()));
	QObject::connect(m_binaryMode, SIGNAL(clicked()), 
					this, SLOT(setBinaryMode()));
	QObject::connect(m_normalMode, SIGNAL(clicked()), 
					this, SLOT(setNormalMode()));
	QObject::connect(m_normalFactor, SIGNAL(clicked()), 
					this, SLOT(setNormalFactor()));
	QObject::connect(m_factorTwo, SIGNAL(clicked()), 
					this, SLOT(setFactorTwo()));
	QObject::connect(m_factorFour, SIGNAL(clicked()), 
					this, SLOT(setFactorThree()));
	QObject::connect(m_mapViewer, SIGNAL(clickedVoxel(int , int)),
					this, SLOT(getVoxelValue(int , int)));
	QObject::connect(m_mapViewer, SIGNAL(updateThresholdLabel(int )),
					this, SLOT(updateBinaryLabel(int )));
	QObject::connect(m_mapViewer, SIGNAL(clickedUpdateVolumericMapMask(int , int, bool)),
					this, SLOT(updateVolumericMapMask(int , int, bool)));
}
/**
 * Initialises the MapEditor widget i.e. creates the ScrollBars, radio button, etc,
 * and lays them out in layout mangers.
 */
void MapEditor::initialise() {
	
	QVBoxLayout *bottomLayout = new QVBoxLayout;
	bottomLayout->setSpacing (10);
	bottomLayout->setResizeMode (QLayout::Fixed);
	QButtonGroup *planeSelection = new QButtonGroup(2, Qt::Vertical, tr("Plane & Cross Section Selection"), this);
	
	QButtonGroup *planeGroup = new QButtonGroup(3, Qt::Horizontal, tr("Planes"), planeSelection);
	m_xPlane = new QRadioButton(tr("X"), planeGroup);
	m_yPlane = new QRadioButton(tr("Y"), planeGroup);
	m_zPlane = new QRadioButton(tr("Z "), planeGroup);
	m_yPlane->setChecked(true);
	
	QButtonGroup *scrollBarGroup = new QButtonGroup(2, Qt::Horizontal, tr("Cross Sections"), planeSelection);
	m_planeScrollBar = new QScrollBar(Qt::Horizontal, scrollBarGroup);	
	m_planeScrollBar->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, m_planeScrollBar->sizePolicy().hasHeightForWidth() ) );
	m_planeScrollBar->setRange(0, m_currentVolumericMap->getSizeY()-1);
	m_planeScrollBar->setValue(m_currentVolumericMap->getSizeY()/2);
	m_planeScrollBar->setPageStep(1);
	m_planeScrollBar->setLineStep(1);
	m_planeString = QString::number(m_planeScrollBar->value(), 10);
	m_planeLabel = new QLabel(tr(m_planeString),scrollBarGroup);
	m_planeLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, m_planeLabel->sizePolicy().hasHeightForWidth() ) );
	m_voxelLabel = new QLabel(tr(m_voxelString),this);
	
	bottomLayout->addWidget(m_voxelLabel);
	bottomLayout->addWidget(planeSelection);

	QButtonGroup *projectionGroup = new QButtonGroup(3, Qt::Horizontal, tr("Plane Projection"), this);
	m_xProjection = new QRadioButton(tr("X Projection"), projectionGroup);
	m_yProjection = new QRadioButton(tr("Y Projection"), projectionGroup);
	m_zProjection = new QRadioButton(tr("Z Projection"), projectionGroup);
	bottomLayout->addWidget(projectionGroup);
	
	QButtonGroup *display = new QButtonGroup(1, Qt::Horizontal, tr("Display"), this);

	QButtonGroup *displayModeGroup = new QButtonGroup(2, Qt::Horizontal, tr("Modes"), display);
	m_binaryMode = new QRadioButton(tr("Binary"), displayModeGroup);
	m_normalMode = new QRadioButton(tr("Normal"), displayModeGroup);
	m_normalMode->setChecked(true);
	
	QButtonGroup *binaryGroup = new QButtonGroup(2, Qt::Horizontal, tr("Binary Threshold"), display);
	m_binaryScrollBar = new QScrollBar(Qt::Horizontal, binaryGroup);
	m_binaryScrollBar->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, m_binaryScrollBar->sizePolicy().hasHeightForWidth() ) );
	m_binaryScrollBar->setRange(1, 9);
	m_binaryScrollBar->setValue(5);
	m_binaryScrollBar->setPageStep(1);
	m_binaryScrollBar->setLineStep(1);
	m_binaryScrollBar->setEnabled(false);
	m_binaryString = QString::number(m_binaryScrollBar->value() * 10, 10);
	m_binaryString.append(" %");
	m_binaryLabel = new QLabel(tr(m_binaryString),binaryGroup);
	m_binaryLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, m_binaryLabel->sizePolicy().hasHeightForWidth() ) );
	
	QButtonGroup *gammaGroup = new QButtonGroup(2, Qt::Horizontal, tr("Gamma Adjustment"), display);
	m_gammaScrollBar = new QScrollBar(Qt::Horizontal, gammaGroup);
	m_gammaScrollBar->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, m_gammaScrollBar->sizePolicy().hasHeightForWidth() ) );
	m_gammaScrollBar->setRange(1, 30);
	m_gammaScrollBar->setValue(10);
	m_gammaScrollBar->setPageStep(1);
	m_gammaScrollBar->setLineStep(1);
	double value = 1.0;
	m_gammaString = QString("%1" )
                    .arg( value, 0, 'f' , 1 );
	m_gammaLabel = new QLabel(tr(m_gammaString),gammaGroup);
	m_gammaLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, m_gammaLabel->sizePolicy().hasHeightForWidth() ) );
	bottomLayout->addWidget(display);
	
	QButtonGroup *factorGroup = new QButtonGroup(3, Qt::Horizontal, tr("Factor Size"), this);
	m_normalFactor = new QRadioButton(tr("Normal"), factorGroup);
	m_factorTwo = new QRadioButton(tr("Factor 2"), factorGroup);
	m_factorFour = new QRadioButton(tr("Factor 3"), factorGroup);
	m_normalFactor->setChecked(true);
	bottomLayout->addWidget(factorGroup);
	
	QButtonGroup *zoomGroup = new QButtonGroup(3, Qt::Vertical, tr("Zoom"), this);
	m_zoomScrollBar = new QScrollBar(Qt::Horizontal, zoomGroup);
	m_zoomScrollBar->setRange(1, 64);
	m_zoomScrollBar->setValue(4);
	m_zoomScrollBar->setPageStep(1);
	m_zoomScrollBar->setLineStep(1);
	bottomLayout->addWidget(zoomGroup);
	
	QHBoxLayout *topLayout= new QHBoxLayout;
	QScrollView *scrollView =  new QScrollView(this);
	m_mapViewer = new MapViewer(scrollView);
	scrollView->addChild(m_mapViewer);
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeZ(), m_currentVolumericMap->getSizeX(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
	topLayout->addWidget(scrollView);
	
	QHBoxLayout *mainLayout= new QHBoxLayout(this);
	mainLayout->setSpacing (10);
	mainLayout->insertLayout(0, topLayout, 20);
	mainLayout->insertLayout(1, bottomLayout, 0);
}
/**
 * Gets, the cross section of the currently active plane at a specific coordinate.  Updates
 * the label to inform the user at which particular slice they are at in the cross section. 
 * \param coordinate specifies the slice in the current plane 
 */
void MapEditor::getNewCrossSection(int coordinate) {

	m_planeString = QString::number(m_planeScrollBar->value(), 10);
	m_planeLabel->setText(m_planeString);
	if(m_xPlane->isChecked()) {
		getCrossSection('x',coordinate);
		m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
					m_currentVolumericMap->getSizeY(), m_currentVolumericMap->getSizeZ(), 
					m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),m_binaryMode->isChecked());

	} 
	if(m_yPlane->isChecked()) {
		getCrossSection('y',coordinate);
		m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
					m_currentVolumericMap->getSizeZ(), m_currentVolumericMap->getSizeX(),
					m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
	}
	if(m_zPlane->isChecked()) {
		getCrossSection('z',coordinate);
		m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
					m_currentVolumericMap->getSizeX(), m_currentVolumericMap->getSizeY(),
					m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
	}
}
/**
 * Sets up the x-plane view when the x-plane radio button is activated 
 */
void MapEditor::setXPlaneView() {
	disableProjections();
	m_planeScrollBar->setRange(0, m_currentVolumericMap->getSizeX()-1);
	m_currentPlane = 'x';
	getCrossSection(m_currentPlane, getCoordinate());
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeY(), m_currentVolumericMap->getSizeZ(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
	
}
/**
 * Sets up the y-plane view when the y-plane radio button is activated 
 */
void MapEditor::setYPlaneView() {
	disableProjections();
	m_planeScrollBar->setRange(0, m_currentVolumericMap->getSizeY()-1);
	m_currentPlane = 'y';
	getCrossSection(m_currentPlane, getCoordinate());
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeZ(), m_currentVolumericMap->getSizeX(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
}
/**
 * Sets up the z-plane view when the z-plane radio button is activated 
 */
void MapEditor::setZPlaneView() {
	disableProjections();
	m_planeScrollBar->setRange(0, m_currentVolumericMap->getSizeZ()-1);
	m_currentPlane = 'z';
	getCrossSection(m_currentPlane, getCoordinate());
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeX(), m_currentVolumericMap->getSizeY(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
}
/**
 * Sets up the x-projection view when the x-projection radio button is activated 
 */
void MapEditor::setXProjectionView() {
	disablePlanes();
	m_currentPlane = 'x';
	getProjection(m_currentPlane);
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeY(), m_currentVolumericMap->getSizeZ(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
}
/**
 * Sets up the y-projection view when the y-projection radio button is activated 
 */
void MapEditor::setYProjectionView() {
	disablePlanes();
	m_currentPlane = 'y';
	getProjection(m_currentPlane);
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeZ(), m_currentVolumericMap->getSizeX(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
}
/**
 * Sets up the z-projection view when the z-projection radio button is activated 
 */
void MapEditor::setZProjectionView() {
	disablePlanes();
	m_currentPlane = 'z';
	getProjection(m_currentPlane);
	m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
				m_currentVolumericMap->getSizeX(), m_currentVolumericMap->getSizeY(),
				m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
}
/**
 * Sets up the display to binary mode. 
 */ 
void MapEditor::setBinaryMode() {
	m_binaryScrollBar->setEnabled(true);
	m_gammaScrollBar->setEnabled(false);
	applyCurrentPlane();
}
/**
 * Sets up the display to normal mode. 
 */ 
void MapEditor::setNormalMode() {
	m_binaryScrollBar->setEnabled(false);
	m_gammaScrollBar->setEnabled(true);
	applyCurrentPlane();
}
/**
 * Sets the factor to normal, therefore each vowel element in the volumeric map recieves one pixel
 * element.
 */
void MapEditor::setNormalFactor() {
	int scrollValue;
	if(m_ordinalVolumericMap != NULL) {
		delete [] m_currentVolumericMap;
		m_currentVolumericMap = m_ordinalVolumericMap;
		m_ordinalVolumericMap = NULL;
		scrollValue = m_planeScrollBar->value() * m_factorValue;
		checkPlaneActivated();
		m_planeScrollBar->setValue(scrollValue);
	} 
}
/**
 * Sets the factor to two, therefore eight vowel elements in the volumeric map are averaged
 * together and recieve one pixel element.
 */
void MapEditor::setFactorTwo() {
	int scrollValue;
	setFactorValue(2);
	if(m_ordinalVolumericMap == NULL) {
		m_ordinalVolumericMap = m_currentVolumericMap;
		m_currentVolumericMap = m_ordinalVolumericMap->calculateAverageVolumericMap(m_factorValue);
		scrollValue = static_cast <int> (m_planeScrollBar->value() / m_factorValue + 0.5);
		checkPlaneActivated();
		m_planeScrollBar->setValue(scrollValue);
		
	} else {
		delete [] m_currentVolumericMap;
		m_currentVolumericMap = m_ordinalVolumericMap->calculateAverageVolumericMap(m_factorValue);
		scrollValue = static_cast <int> (m_planeScrollBar->value() / m_factorValue + 0.5);
		checkPlaneActivated();
		m_planeScrollBar->setValue(scrollValue);
	}
}
/**
 * Sets the factor to three, therefore 27 vowel elements in the volumeric map are averaged
 * together and recieve one pixel element.
 */
void MapEditor::setFactorThree() {
	int scrollValue;
	setFactorValue(3);
	if(m_ordinalVolumericMap == NULL) {
		m_ordinalVolumericMap = m_currentVolumericMap;
		m_currentVolumericMap = m_ordinalVolumericMap->calculateAverageVolumericMap(m_factorValue);
		scrollValue = static_cast <int> (m_planeScrollBar->value() / m_factorValue + 0.5);
		checkPlaneActivated();
		m_planeScrollBar->setValue(scrollValue);
	} else {
		delete [] m_currentVolumericMap;
		m_currentVolumericMap = m_ordinalVolumericMap->calculateAverageVolumericMap(m_factorValue);
		scrollValue = static_cast <int> (m_planeScrollBar->value() / m_factorValue + 0.5);
		checkPlaneActivated();
		m_planeScrollBar->setValue(scrollValue);
	}
}
/**
 * Sets the current factor size.
 * \param factorValue the new factor value.
 */
void MapEditor::setFactorValue(int factorValue) {
	m_factorValue = factorValue;
}
/**
 * Gets the current plane that is activated on the MapViewer.
 * \return m_currentPlane current plane;
 */
char MapEditor::getCurrentPlane() {
	return m_currentPlane;
}
/**
 * Turns the x, y and z plane radio buttons off and disables the plane ScrollBar.  Called when the user
 * wishes to see a projection view of the volume set.
 */
void MapEditor::disablePlanes() {
	m_xPlane->setChecked(false);
	m_yPlane->setChecked(false);
	m_zPlane->setChecked(false);
	m_planeScrollBar->setEnabled(false);
}
/**
 * Turns the x, y and z projection radio buttons off.  Called when the user wishes to 
 * iterate through the plane slices of the volume set.
 */
void MapEditor::disableProjections() {
	m_xProjection->setChecked(false);
	m_yProjection->setChecked(false);
	m_zProjection->setChecked(false);
	m_planeScrollBar->setEnabled(true);
}
/** 
 * Sets the current plane on the MapViewer.
 */ 
void MapEditor::applyCurrentPlane() {
	switch(getCurrentPlane()) {
		case 'x':
			m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(), 
						m_currentVolumericMap->getSizeY(), m_currentVolumericMap->getSizeZ(),
						m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
		break;
		case 'y':
			m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(),
						m_currentVolumericMap->getSizeZ(), m_currentVolumericMap->getSizeX(),
						m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
		break;
		case 'z':
			m_mapViewer->createVolumericMap(getCrossSectionVolumericMap(), getCrossSectionVolumericMapMask(), 
						m_currentVolumericMap->getSizeX(), m_currentVolumericMap->getSizeY(),
						m_currentVolumericMap->getMaxDensity(), m_currentVolumericMap->getMinDensity(),	m_binaryMode->isChecked());
		break;
	}
}
/**
 * Gets the current value of the m_planeScrollBar.
 * \return the current value the m_planeScrollBar is set to.
 */
int MapEditor::getCoordinate() {
	return m_planeScrollBar->value();
}
/**
 * pindex finds the cross section of a 3-D slice.
 * \param x is the x coordinate of the 3-D slice
 * \param y is the y coordinate of the 3-D slice
 * \param z is the z coordinate of the 3-D slice
 * \return the data value at that 3-D cross section
 */
unsigned int MapEditor::pindex (int x, int y, int z) {
	return x + y * m_currentVolumericMap->getSizeX() + z* m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeY();
	
}
/**
 * Computes a new cross section from the ordinal data set and data set mask.
 * \param planeFlag is the cross section plane 
 * \param coordinate is the coordinate of the cross section as function of plane
 */
void MapEditor::getCrossSection(char planeFlag, int coordinate) {

	int x, y, z;
	if (m_crossSectionVolumericMap != NULL ) {
		delete[] m_crossSectionVolumericMap;
	}
	if (m_crossSectionVolumericMapMask != NULL ) {
		delete[] m_crossSectionVolumericMapMask;
	}
	switch(planeFlag) {
		case 'x':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeY() * m_currentVolumericMap->getSizeZ()];
			m_crossSectionVolumericMapMask = (char *) new char[m_currentVolumericMap->getSizeY() * m_currentVolumericMap->getSizeZ()];
			m_coordinate = coordinate;
			for(z = 0; z < m_currentVolumericMap->getSizeZ(); z++) {
				for(y = 0; y < m_currentVolumericMap->getSizeY(); y++) {
					m_crossSectionVolumericMap[z*m_currentVolumericMap->getSizeY()+y] = m_currentVolumericMap->getValue(m_coordinate, y, z);
					m_crossSectionVolumericMapMask[z*m_currentVolumericMap->getSizeY()+y] = m_ordinalVolumericMapMask[pindex(m_coordinate, y, z)];
				}
			}
		break;
		case 'y':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeZ()];
			m_crossSectionVolumericMapMask = (char *) new char[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeZ()];
			m_coordinate = coordinate;
			for(x = 0; x < m_currentVolumericMap->getSizeX(); x++) {
				for(z = 0; z < m_currentVolumericMap->getSizeZ(); z++) {
					m_crossSectionVolumericMap[x*m_currentVolumericMap->getSizeZ()+z] = m_currentVolumericMap->getValue(x, m_coordinate, z);
					m_crossSectionVolumericMapMask[x*m_currentVolumericMap->getSizeZ()+z] = m_ordinalVolumericMapMask[pindex(x, m_coordinate, z)];
				}
			}
		break;
		case 'z':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeY()];
			m_crossSectionVolumericMapMask = (char *) new char[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeY()];
			m_coordinate = coordinate;
			for(y = 0; y < m_currentVolumericMap->getSizeY(); y++) {
				for(x = 0; x < m_currentVolumericMap->getSizeX(); x++) {
					m_crossSectionVolumericMap[y*m_currentVolumericMap->getSizeX()+x] = m_currentVolumericMap->getValue(x, y, m_coordinate);
					m_crossSectionVolumericMapMask[y*m_currentVolumericMap->getSizeX()+x] = m_ordinalVolumericMapMask[pindex(x, y, m_coordinate)];
				}
			}
		break;
		default:
			m_crossSectionVolumericMap = NULL;
			m_crossSectionVolumericMapMask = NULL;
		break;
			
	}
}
/**
 * Computes a new projection from the ordinal data set and data set mask.
 * \param projectionFlag is the projection plane  
 */
void MapEditor::getProjection(char projectionFlag) {
	int x, y, z;
	double value = 0.0;
	if (m_crossSectionVolumericMap != NULL ) {
		delete[] m_crossSectionVolumericMap;
	}
	if (m_crossSectionVolumericMapMask != NULL ) {
		delete[] m_crossSectionVolumericMapMask;
	}
	m_crossSectionVolumericMapMask = NULL;
	
	switch(projectionFlag) {
		case 'x':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeY() * m_currentVolumericMap->getSizeZ()];
			for(z = 0; z < m_currentVolumericMap->getSizeZ(); z++) 
				for(y = 0; y < m_currentVolumericMap->getSizeY(); y++) {
					for(x = 0; x < m_currentVolumericMap->getSizeX(); x++) 
						value = value + m_currentVolumericMap->getValue(x, y, z);
					m_crossSectionVolumericMap[z*m_currentVolumericMap->getSizeY()+y] = value / m_currentVolumericMap->getSizeX();
					value = 0.0;
				}
		break;
		case 'y':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeZ()];
			for(x = 0; x < m_currentVolumericMap->getSizeX(); x++) 
				for(z = 0; z < m_currentVolumericMap->getSizeZ(); z++) {
					for(y = 0; y < m_currentVolumericMap->getSizeY(); y++) 
						value = value + m_currentVolumericMap->getValue(x, y, z);
					m_crossSectionVolumericMap[x*m_currentVolumericMap->getSizeZ()+z] = value / m_currentVolumericMap->getSizeY();
					value = 0.0;
				}
		break;
		case 'z':
			m_crossSectionVolumericMap = (double *) new double[m_currentVolumericMap->getSizeX() * m_currentVolumericMap->getSizeY()];
			for(y = 0; y < m_currentVolumericMap->getSizeY(); y++) 
				for(x = 0; x < m_currentVolumericMap->getSizeX(); x++) {
					for(z = 0; z < m_currentVolumericMap->getSizeZ(); z++) 
						value = value + m_currentVolumericMap->getValue(x, y, z);
					m_crossSectionVolumericMap[y*m_currentVolumericMap->getSizeX()+x] = value / m_currentVolumericMap->getSizeZ();	
					value = 0.0;
			}
		break;
	}
}
/**
 * Gets the current cross section data set.
 * \return _crossSectionVolumericMap is the current cross section data set
 */
double* MapEditor::getCrossSectionVolumericMap() {
	return m_crossSectionVolumericMap;
}
/**
 * Gets the current cross section data set mask, which is used for editing the of the data set
 * \return m_crossSectionVolumericMapMask is the data set mask
 */
char* MapEditor::getCrossSectionVolumericMapMask() {
	return m_crossSectionVolumericMapMask;
}
/**
 * Gets the voxel value at the corresponding position in the MapViewer
 * \param positionsX is the x coordinate in the data set
 * \param positionsY is the y coordinate in the data set
 */
void MapEditor::getVoxelValue(int positionsX, int positionsY) {
	checkPlaneActivated(positionsX, positionsY);
	if(m_xPlane->isChecked() || m_yPlane->isChecked() || m_zPlane->isChecked()) {
		m_voxelValue = m_currentVolumericMap->getValue(m_positionX, m_positionY, m_positionZ);
		m_voxelString = QString("The voxel value at x:%1 y:%2 z:%3 is:%4")
						.arg(m_positionX)
						.arg(m_positionY)
						.arg(m_positionZ)
						.arg(m_voxelValue);
		m_voxelLabel->setText(m_voxelString); 
	} else {
		m_voxelString = QString("The voxel value at x:%1 y:%2 z:%3 is:%4")
						.arg(m_positionX)
						.arg(m_positionY)
						.arg(m_positionZ)
						.arg(m_voxelValue);
		m_voxelLabel->setText(m_voxelString); 
	}
}
/** 
 * Update the label to inform the user of the current percentage of the threshold value.
 * \param thresholdValue, the new value that the binary threshold is set to.
 */
void MapEditor::updateBinaryLabel(int thresholdValue) {
	m_binaryString = QString::number(thresholdValue * 10, 10);
	m_binaryString.append(" %");
	m_binaryLabel->setText(m_binaryString); 
}
/** 
 * Update the label to inform the user of the current gamma value.
 * \param gammaValue, the new value that the gamma is set to.
 */
void MapEditor::updateGammaLabel(int gammaValue) {
	double value = gammaValue / 10.0f; 
	m_gammaString = QString("%1" )
                    .arg( value, 0, 'f' , 1 );
	m_gammaLabel->setText(m_gammaString); 
}
/**
 * Updates the data set mask at the corresponding position.
 * \param x, the x position of the mouse
 * \param y, the y position of the mouse
 * \param value, the mask needs to be set to either a one or a zero.
 */
void MapEditor::updateVolumericMapMask(int x, int y, bool value) {
	checkPlaneActivated(x, y);
	m_ordinalVolumericMapMask[pindex(m_positionX, m_positionY, m_positionZ)] = value;
}
/**
 * Checks to indentify which plane is currently activated on the MapEditor. Use to update
 * voxel values within the data set and data set mask.
 * \param x, the x position within the data set/mask
 * \param y, the y position within the data set/mask
 */
void MapEditor::checkPlaneActivated(int x, int y) {
	if(m_xPlane->isChecked()) {
		m_positionY = x;
		m_positionZ = y;
		m_positionX = getCoordinate();
	}
	if(m_yPlane->isChecked()) {
		m_positionX = y;
		m_positionZ = x;
		m_positionY = getCoordinate();
	}
	if(m_zPlane->isChecked()) {
		m_positionX = x;
		m_positionY = y;
		m_positionZ = getCoordinate();
	}
	if(m_xProjection->isChecked()) {
		m_positionY = x;
		m_positionZ = y;
		m_positionX = 0;
		m_voxelValue = m_mapViewer->getValue(m_positionY, m_positionZ);
	}
	if(m_yProjection->isChecked()) {
		m_positionX = y;
		m_positionZ = x;
		m_positionY = 0;
		m_voxelValue = m_mapViewer->getValue(m_positionZ, m_positionX);
	}
	if(m_zProjection->isChecked()) {
		m_positionX = x;
		m_positionY = y;
		m_positionZ = 0;
		m_voxelValue = m_mapViewer->getValue(m_positionX, m_positionY);
	}
}

/**
 * Checks to indentify which plane is currently activated on the MapEditor. Use when the factor 
 * value has changed.
 */
void MapEditor::checkPlaneActivated() {
	if(m_xPlane->isChecked()) 
		setXPlaneView();
	if(m_yPlane->isChecked()) 
		setYPlaneView();
	if(m_zPlane->isChecked())
		setZPlaneView();
	if(m_xProjection->isChecked())
		setXProjectionView();
	if(m_yProjection->isChecked())
		setYProjectionView();
	if(m_zProjection->isChecked())
		setZProjectionView();
}

