#ifndef MAPEDITOR_H
#define MAPEDITOR_H

#include <qscrollbar.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qstring.h>

#include "MapViewer.h"
#include "svt_volume.h"
#include "svt_basics.h"

class DLLEXPORT MapEditor : public QWidget {
	Q_OBJECT
	
	public:
		/**
		 * MapEditor class extends QWidget and acts as a parent widget for QSliders, QRadioButtons, QLabels,
		 * etc.  It is used as a controller class between a MapViewer object and a CrossSection object.  It uses
		 * it's child widgets to trigger events such that a new slice or plane can be calculated and 
		 * displayed in the MapViewer widget.
		 */
		MapEditor(svt_volume<double> *volumericMap, QWidget *parent = 0, const char *name = 0, WFlags flags = 0);
		/**
		 * Initialises the MapEditor widget i.e. creates the sliders, radio button, etc,
		 * and lays them out in layout mangers.
		 */
		void initialise();
		/**
		 * Sets the current factor size.
		 * \param factorValue the new factor value.
		 */
		void setFactorValue(int ); 
		/**
		 * Gets the current plane that is activated on the MapViewer.
		 * \return m_currentPlane current plane;
		 */
		char getCurrentPlane();
		/** 
		 * Gets the current value of the _planeSlider.
		 * \return the current value the _planeSlider is set to.
		 */
		int getCoordinate();
		/**
		 * pindex finds the cross section of a 3-D slice.
		 * \param x is the x coordinate of the 3-D slice
		 * \param y is the y coordinate of the 3-D slice
		 * \param z is the z coordinate of the 3-D slice
		 * \return the data value at that 3-D cross section
		 */
		unsigned int pindex (int , int , int );
		/**
	 	 * Computes a new cross section from the ordinal data set.
		 * \param planeFlag is the cross section plane 
		 * \param coordinate is the coordinate of the cross section as function of plane
		 */
		void getCrossSection(char , int );
		/**
		 * Computes a new projection from the ordinal data set and data set mask.
		 * \param projectionFlag is the projection plane 
		 */
		void MapEditor::getProjection(char );
		/**
		 * Gets the current cross section data set.
		 * \return _crossSectionVolumericMap is the current cross section data set
		 */
		double* getCrossSectionVolumericMap();
		/** 
		 * Gets the current cross section data set mask, which is used for editing the of the data set
		 * \return m_ordinalVolumericMapMask is the data set mask
		 */
		char* getCrossSectionVolumericMapMask();

	public slots:	
		/** 
		 * Gets, the cross section of the currently active plane at a specific coordinate.  Updates
		 * the label to inform the user at which particular slice they are at in the cross section. 
		 * \param coordinate specifies the slice in the current plane 
		 */
		void getNewCrossSection(int );
		/** 
		 *Sets up the x-plane view when the x-plane radio button is activated 
		 */
		void setXPlaneView();
		/** 
		 * Sets up the y-plane view when the y-plane radio button is activated 
		 */
		void setYPlaneView();
		/** 
		 * Sets up the z-plane view when the z-plane radio button is activated 
		 */
		void setZPlaneView();
		/**
		 * Sets up the x-projection view when the x-projection radio button is activated 
		 */
		void setXProjectionView(); 
		/**
		 * Sets up the y-projection view when the y-projection radio button is activated 
		 */
		void setYProjectionView(); 
		/**
		 * Sets up the z-projection view when the z-projection radio button is activated 
		 */
		void setZProjectionView(); 
		/**
	 	 * Sets the display to binary mode. 
		 */ 
		void setBinaryMode();
		/** 
		 * Sets the display to normal mode. 
		 */ 
		void setNormalMode();
		/**
		 * Sets the factor to normal, therefore each vowel element in the volumeric map recieves one pixel
		 * element.
		 */
		void setNormalFactor();
		/**
		 * Sets the factor to two, therefore eight vowel elements in the volumeric map are averaged
		 * together and recieve one pixel element.
		 */
		void setFactorTwo();
		/**
		 * Sets the factor to three, therefore 27 vowel elements in the volumeric map are averaged
		 * together and recieve one pixel element.
		 */
		void setFactorThree();	
		/**
	 	 * Gets the voxel value at the corresponding position in the MapViewer
		 * \param x is the x coordinate in the data set
		 * \param y is the y coordinate in the data set
		 */
		void getVoxelValue(int , int  );
		/** 
		 * Update the label to inform the user of the current percentage of the threshold value.
		 * \param thresholdValue, the new value that the binary threshold is set to.
		 */
		void updateBinaryLabel(int ); 
		/** 
		 * Update the label to inform the user of the current gamma value.
		 * \param gammaValue, the new value that the gamma is set to.
		 */
		void updateGammaLabel(int ); 
		/**
		 * Updates the data set mask at the corresponding position.
		 * \param x, the x position of the mouse
		 * \param y, the y position of the mouse
		 * \param value, the mask needs to be set to either a one or a zero.
		 */
		void updateVolumericMapMask(int , int , bool );

	private:
		/**
		 *Checks to indentify which plane is currently activated on the MapEditor. Use to update
		 * voxel values within the data set and data set mask.
		 * \param x, the x position within the data set/mask
		 * \param y, the y position within the data set/mask
		 */
		void checkPlaneActivated(int , int ); 
		/**
		 * Checks to indentify which plane is currently activated on the MapEditor. Use when the factor 
		 * value has changed.
		 */
		void checkPlaneActivated();
		/** 
		 * Sets the current plane on the MapViewer.
		 */ 
		void applyCurrentPlane();
		/**
		 * Turns the x, y and z plane radio buttons off and disables the plane slider.  Called when 
		 * the user wishes to see a projection view of the volume set.
		 */
		void disablePlanes();
		/**
		 * Turns the x, y and z projection radio buttons off.  Called when the user wishes to 
		 * iterate through the plane slices of the volume set.
		 */
		void disableProjections();
		
		svt_volume<double> *m_ordinalVolumericMap;
		svt_volume<double> *m_currentVolumericMap;
		double *m_crossSectionVolumericMap;
		char *m_ordinalVolumericMapMask;
		char *m_crossSectionVolumericMapMask;
		char m_currentPlane;
		int m_coordinate;
		int m_positionX;
		int m_positionY;
		int m_positionZ;
		int m_factorValue;
		double m_voxelValue;
		QLabel *m_planeLabel;
		QString m_voxelString;
		QLabel *m_voxelLabel;
		QString m_planeString;
		QLabel *m_binaryLabel;
		QString m_binaryString;
		QLabel *m_gammaLabel;
		QString m_gammaString;
		QScrollBar *m_zoomScrollBar;
		QScrollBar *m_planeScrollBar;
		QScrollBar *m_gammaScrollBar;
		QScrollBar *m_binaryScrollBar;
		QRadioButton *m_xPlane;
		QRadioButton *m_yPlane;
		QRadioButton *m_zPlane;
		QRadioButton *m_xProjection;
		QRadioButton *m_yProjection;
		QRadioButton *m_zProjection;
		QRadioButton *m_binaryMode;
		QRadioButton *m_normalMode;
		QRadioButton *m_normalFactor;
		QRadioButton *m_factorTwo;
		QRadioButton *m_factorFour;
		MapViewer *m_mapViewer;
		
};

#endif
