#include <qpainter.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <math.h>

#include "MapViewer.h"

/**
 * MapViewer extends QWidget, and overrides the paintEvent(), setPenColor() and sizeHint()
 * functions associated with QWidget.  MapViewer takes a 2D-array and renders the arrays contents
 * to produce a black and white map to the screen.  This is done by calling the createDataSet() function.
 * \param *parent is the parent QWidget that MapViewer resides in
 * \param *name is some text that can be used to identify the MapViewer from other QObjects.
 */ 
MapViewer::MapViewer(QWidget *parent, const char *name, WFlags flags) : QWidget(parent, name, flags | WNoAutoErase) {
	m_zoomMin = 2;
	m_zoomMax = 64;
    m_zoomScale = 4;
	m_gammaValue = 1;
	m_thresholdScale = 5;
	m_dataSet = NULL;
	m_dataSetMask = NULL;
	setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    m_curColor = black;
    setZoomScale(m_zoomScale);
	setBackgroundMode (NoBackground);
}
/**
 * Reimplementation from QWidget, returns the ideal size of the MapViewer
 */
QSize MapViewer::sizeHint() const {
    QSize size = QSize(m_sizeX * m_zoomScale, m_sizeY * m_zoomScale);
    if (m_zoomScale >= 3)
        size += QSize(1, 1);
    return size;
}
/**
 * Sets the current pen color
 * \param newColor the pen color
 */
void MapViewer::setPenColor(const QColor &newColor) {
    m_curColor = newColor;
}
/**
 * Sets the zoom factor of the viewer
 * \param newZoom the new zoom scale
 */
void MapViewer::setZoomScale(int newZoom) {
	m_zoomScale = newZoom;
	update();
	updateGeometry();
}
/**
 * Sets the gamma value for the gamma curve.
 * \param gammaValue the new gamma value
 */
void MapViewer::setGammaCurve(int gammaValue) {
	int temp = gammaValue;
	m_gammaValue = (float)(temp) / 10.0f;
	update();
	updateGeometry();
}
/**
 * Sets the binary threshold value for binary mode
 * \param newZoom the new zoom scale
 */
void MapViewer::setBinaryThreshold(int newThreshold) {
	m_thresholdScale = newThreshold;
	emit updateThresholdLabel(m_thresholdScale);
	update();
	updateGeometry();
}
/**
 * Reimplementation from QWidget, to force the MapViewer to paint itself
 * \param Automatically generated event by the application
 */
void MapViewer::paintEvent(QPaintEvent *event) {
	static QPixmap pixmap;
	QRect rect = event->rect();
	
	QSize newSize = rect.size().expandedTo(pixmap.size());
	pixmap.resize(newSize);
	pixmap.fill(this, rect.topLeft());
	
	QPainter painter(&pixmap, this);
	painter.translate(-rect.x(), -rect.y());
    
	if (getZoomFactor() >= 3) {
        painter.setPen(colorGroup().foreground());
        for (int x = 0; x < m_sizeX; ++x)
            painter.drawLine(getZoomFactor() * x, 0,
                             getZoomFactor() * x, getZoomFactor() * m_sizeY);
        for (int y = 0; y < m_sizeY; ++y)
            painter.drawLine(0, getZoomFactor() * y,
                             getZoomFactor() * m_sizeX, getZoomFactor() * y);
    }
    for (int x = 0; x < m_sizeX; ++x) {
		for (int y = 0; y < m_sizeY; ++y)
			drawImagePixel(&painter, x, y);
    }
	bitBlt(this, rect.x(), rect.y(), &pixmap, 0, 0, rect.width(), rect.height());
}
/**
 * Reimplementation from QWidgetm to force the MapViewer to trigger a mouse 
 * press event
 * \param Automatically generated event went a mouse button is pressed
 */
void MapViewer::mousePressEvent(QMouseEvent *event) {
	if(event->button() == RightButton)
		voxelXYCoord(event->pos());
	if(event->button() == LeftButton)
		updateDataSetMask(event->pos());
}
/**
 * Gets the mouse position in the MapViewer as a point.
 * \param position, the position of the mouse on the MapViewer
 */
void MapViewer::voxelXYCoord(const QPoint &position) {
	int x = position.x() / getZoomFactor();
	int y = position.y() / getZoomFactor();
	int newYPosition;
	newYPosition = (m_sizeY - 1) - y;   // because of the coordinate system 
									//i.e. (0,0) is the top left on a QWidget this makes it bottom left
	emit clickedVoxel(x, newYPosition);
}
/**
 * Gets the mouse position in the MapViewer as a point.  This point is used to determine if value
 * at that location within the data set mask should be set to a zero or a one.
 * \param position, the position of the mouse on the MapViewer
 */
void MapViewer::updateDataSetMask(const QPoint &position) {
	int x = position.x() / getZoomFactor();
	int y = position.y() / getZoomFactor();
	int newYPosition;
	bool value;
	newYPosition = (m_sizeY - 1) - y;   // because of the coordinate system 
									//i.e. (0,0) is the top left on a QWidget this makes it bottom left
	if (m_dataSetMask != NULL && x < m_sizeX && newYPosition < m_sizeY && m_sizeX >= 0 && m_sizeY >= 0) {
		if(m_dataSetMask[pindex(x,newYPosition)] == 0) {
			m_dataSetMask[pindex(x,newYPosition)] = 1;
			value = true;
		} else {
			m_dataSetMask[pindex(x,newYPosition)] = 0;
			value = false;
		}
		emit clickedUpdateVolumericMapMask(x, newYPosition, value);
		QPainter painter(this);
		drawImagePixel(&painter, x , newYPosition);
	}
}
/** 
 * Draws a zoomed pixel using a QPainter and also determines if a pixel needs a border around it
 * to indicate that it was selected.
 * \param x is the x coordinate in the data set
 * \param y is the y coordinate in the data set
 */
void MapViewer::drawImagePixel(QPainter *painter, int x, int y) {
	
	int color = getColorValue(x, y);
    int newYPosition; 
	newYPosition = (m_sizeY - 1) - y;   // because of the coordinate system 
									// i.e. (0,0) is the top left on a QWidget this makes it bottom left
	if (getZoomFactor() >= 4) {
		if(m_dataSetMask != NULL && m_dataSetMask[pindex(x,y)] == 1) {
			painter->fillRect(getZoomFactor() * x + 1, getZoomFactor() * newYPosition + 1,
                          getZoomFactor(), getZoomFactor(), QColor(red));
			painter->fillRect(getZoomFactor() * x + 2, getZoomFactor() * newYPosition + 2,
                          getZoomFactor() - 2, getZoomFactor() - 2, QColor(color,color,color));
		} else {
			painter->fillRect(getZoomFactor() * x + 1, getZoomFactor() * newYPosition + 1,
                          getZoomFactor() - 1, getZoomFactor() - 1, QColor(color,color,color));
		}
	} else {
		if(m_dataSetMask != NULL && m_dataSetMask[pindex(x,y)] == 1) {
			painter->fillRect(getZoomFactor() * x + 1, getZoomFactor() * newYPosition + 1,
                          getZoomFactor(), getZoomFactor(), QColor(red));
			painter->fillRect(getZoomFactor() * x + 2, getZoomFactor() * newYPosition + 2,
                          getZoomFactor() - 2, getZoomFactor() - 2, QColor(color,color,color));
		} else {
			painter->fillRect(getZoomFactor() * x, getZoomFactor() * newYPosition,
                          getZoomFactor(), getZoomFactor(), QColor(color, color, color));
		}
	}
}
/** 
 * Gets a value from the data set and scales the value to match the color palette.  If binary mode
 * is selected then density values below the threshold value are 255  and above are 0.
 * \param x is the x coordinate in the data set
 * \param y is the y coordinate in the data set
 * \return a value that will be used in the construction of a QColor
 */
int MapViewer::getColorValue(int x, int y) {
	int colorValue;
	float pixelValue;
	float newPixelValue;
	if (m_viewingMode) {
		double maxDensity = getMaxDensity();
		double threshold = maxDensity * m_thresholdScale / 10;
		double value = getValue(x, y);
		if (threshold < value) {
			colorValue = 255;
			return colorValue;
		} else {
			colorValue = 0;
			return colorValue;
		}
	} else {
		if (m_colorFactorValue <= 0.0f) {
			colorValue = 0;
		} else {
			pixelValue = ((getValue(x, y) - m_minDensity) / m_colorFactorValue);
			newPixelValue = pow(pixelValue, 1 / m_gammaValue);
			colorValue = static_cast <int> (newPixelValue * 255 + 0.5);
		}
		return colorValue;
	}
}
/**
 Creates a new data set.  ATTENDION: This function will delete the old volumeric map
 * when called.
 * \param newVolumericMap is the new volumeric map that is set
 * \param newVolumericMapMask is the new volumeric map mask, used for editing the data set
 * \param sizeX x size of the new volume data
 * \param sizeY y size of the new volume data
 * \param maxDensity maximum density of the new volume
 * \param minDensity minimum density of the new volume
 * \param viewingMode determines the mode 
 */
void MapViewer::createVolumericMap(double *newDataSet, char *newDataSetMask, int sizeX, int sizeY,
								double maxDensity, double minDensity,bool viewingMode) {
	m_dataSet = newDataSet;
	m_dataSetMask = newDataSetMask;
	m_sizeX = sizeX;
    m_sizeY = sizeY;
	m_viewingMode = viewingMode;
	m_maxDensity = maxDensity;
	m_minDensity = minDensity;
	m_colorFactorValue = (m_maxDensity - m_minDensity);
	update();
	updateGeometry();
}
/**
 * Get the value at a position inside the data set
 * \param x, x coordinate
 * \param y, y coordinate
 * \return value
*/
double MapViewer::getValue(int x, int y) {
	if (m_dataSet !=NULL && x < m_sizeX && y < m_sizeY && m_sizeX >= 0 && m_sizeY >= 0)
		return m_dataSet[pindex(x,y)];
	else
	return 0.0f;
}

/**
 * Get the value at a position inside the data set
 * \param count counter
 * \return value
 */
double MapViewer::getValue(int count) const {
	if (m_dataSet != NULL )
		return m_dataSet[count];
	else
		return 0.0f;
}

/**
 * Calculate the minimum and maximum density
 */
void MapViewer::calcMinMaxDensity() {
	long int count;
	long int nvox =  m_sizeX * m_sizeY;

	/* compute range of density and set cutoff */
	m_maxDensity = getValue(0,0);
	m_minDensity = getValue(0,0);
	for (count=0;count<nvox;count++) {
		if (m_maxDensity < getValue(count)) m_maxDensity = getValue(count);
         if (m_minDensity > getValue(count)) m_minDensity = getValue(count);
	}
}

/**
 * Gets the minimum density value in the data set
 * \return the minimum density
 */
double MapViewer::getMinDensity() {
	return m_minDensity;
}

/**
 * Gets the maximum density value in the data set
 * \return the maximum density
 */
double MapViewer::getMaxDensity() {
	return m_maxDensity;
}

/**
 * Sets the minimum density value in the data set
 * \param the minimum density
 */
void MapViewer::setMinDensity(double minDensity) {
	m_minDensity = minDensity;
}
/**
 * Sets the maximum density value in the data set
 * \param the maximum density
 */
void MapViewer::setMaxDensity(double maxDensity) {
	m_maxDensity = maxDensity;
}
/**
 * Gets the color factor value i.e. color value = (max - min) /255
 * \return the color factor value 
 */
 
double MapViewer::getColorFactor() {
	return m_colorFactorValue;
}
/**
 * Gets the index of cross section point in the data set
 * \param x the x coordinate in the data set
 * \param y the y coordinate in the data set
 * \return cross section point
 */
unsigned long MapViewer::pindex(int x, int y) const {
	return  y * m_sizeX + x;
}
/**
 * Sets the zoom to default zoom factor
 */
void MapViewer::applyDefaultZoom() {
	m_zoomScale = 6;
	setZoomScale(m_zoomScale);
}

/**
 * Sets the zoom to minimum zoom factor
 */
void MapViewer::applyMinZoom() {
	m_zoomScale = 1;
	setZoomScale(m_zoomScale);
}

/**
 * Sets the zoom to maximum zoom factor
 */
void MapViewer::applyMaxZoom() {
	m_zoomScale = 64;
	setZoomScale(m_zoomScale);
}
/**
 * Returns the maximum zoom factor the viewer is allowed to go to
 * \return Maximum zoom factor
 */
int MapViewer::getMaxZoomFactor() {
	return m_zoomMax;
}

/**
 * Returns the minimum zoom factor the viewer is allowed to go to
 * \return Minimum zoom factor
 */
int MapViewer::getMinZoomFactor() {
	return m_zoomMin;
}

