/***************************************************************************
                          svt_volume  -  description
                             -------------------
    begin                : Oct 23 2000
    author               : Stefan Birmanns
    email                : s.birmanns@fz-juelich.de
 ***************************************************************************/

#ifndef SVT_VOLUME
#define SVT_VOLUME

/** A container for volume data
  *@author Stefan Birmanns & Paul Boyle
  */
template<class dt>
class DLLEXPORT svt_volume {

protected:
    	dt* m_pData;
    	int m_iSizeX;
    	int m_iSizeY;
    	int m_iSizeZ;
	int m_iNumberVoxels;
	bool m_bCalculateDensity;
    	dt m_fMaxDensity;
    	dt m_fMinDensity;
	dt m_fDensity;

public:

    	/**
     * Constructor
     */
    	svt_volume()
    	{
        	setData(-1,-1,-1,(dt*)NULL);
		m_bCalculateDensity = true;
    	};

    	/**
    	* sets the complete volume data set
     * \param iSizeX x size of the new volume data
     * \param iSizeY y size of the new volume data
     * \param iSizeZ z size of the new volume data
     * \param pData  pointer to the new  volume data
     */
     void setData(int iSizeX, int iSizeY, int iSizeZ, dt* pData)
     {
         m_iSizeX = iSizeX;
         m_iSizeY = iSizeY;
         m_iSizeZ = iSizeZ;
         m_pData = pData;
		m_bCalculateDensity = true;
     };

    	/**
    	* changes one value inside the volume
    	* \param iX x coordinate
    	* \param iY y coordinate
    	* \param iZ z coordinate
    	* \param fValue new value
    	*/
    	void setData(int iX, int iY, int iZ, dt fValue)
     {
         if (iX < m_iSizeX && iY < m_iSizeY && iZ < m_iSizeZ && iX > 0 && iY > 0 && iZ > 0)
             m_pData[pindex(iX,iY,iZ)] = fValue;
		m_bCalculateDensity = true;
     };
	 
    /**
     * changes one value inside the volume
     * \param iX x coordinate
     * \param iY y coordinate
     * \param iZ z coordinate
     * \param fValue new value
     */
     void setValue(int iX, int iY, int iZ, dt fValue)
     {
         setData(iX,iY,iZ, fValue);
		m_bCalculateDensity = true;
     };
	
	/**
	 * Calculates the average volume of the current volume by a factor value. e.g. a factor value of
	 * four, therefore four vowel elements in the volumeric map are averaged together and 
	 * recieve one pixel element.
	 * \param factor the factor to, which the volume will be averaged to. 
	 * \return averageDataSet is the averaged volumeric map determined by the factor value.
	 */
	svt_volume<dt>* calculateAverageVolumericMap(int factor) {
		float factorValue;
		int numVoxels = factor * factor * factor;
		int count = 0; 
		dt value = 0.0f;
		int sizeX = 0;
		int sizeY = 0;
		int sizeZ = 0;
		svt_volume<dt> *averageVolumericMap = new svt_volume<dt>();
		factorValue = 1.0 / factor;

		sizeX = static_cast <int> ((m_iSizeX * factorValue) + 0.5);
		sizeY = static_cast <int> ((m_iSizeY * factorValue) + 0.5);
		sizeZ = static_cast <int> ((m_iSizeZ * factorValue) + 0.5);
		averageVolumericMap->createVolume(sizeX, sizeY, sizeZ);

		for(int indexZ = 0; indexZ < sizeZ; indexZ++) 
			for(int indexY = 0; indexY < sizeY; indexY++) 
				for(int indexX = 0; indexX < sizeX; indexX++) {
					for(int indexZZ = 0; indexZZ < factor; indexZZ++) 
						for(int indexYY = 0; indexYY < factor; indexYY++) 
							for(int indexXX = 0; indexXX < factor; indexXX++)
								value = value + getValue((factor * indexX)+indexXX, (factor * indexY)+indexYY, (factor * indexZ)+indexZZ);
					averageVolumericMap->m_pData[count] = value / numVoxels;
					count++;
					value = 0.0f;
				}
		return averageVolumericMap;
	}
	
	/**
     * get the x size of the volume
     * \return x size of the volume
     */
     int getSizeX()
     {
		return m_iSizeX;
     };
	 
	/**
     * get the y size of the volume
     * \return y size of the volume
     */
     int getSizeY()
     {
         return m_iSizeY;
     };
	 
    /**
     * get the z size of the volume
     * \return z size of the volume
     */
    	int getSizeZ()
    	{
		return m_iSizeZ;
	};

	/**
	 * get the total number of voxels in the volume
	 * \return number of voxels in the volume
	 */
	int getNumberVoxels() {
		return m_iNumberVoxels = m_iSizeX * m_iSizeY * m_iSizeZ;
	}
	
	/**
	 * get the calculate density flag
	 * \return calculate density flag
	 */
	bool getCalculateDensityFlag() {
		return m_bCalculateDensity;
	};

	/**
	 * set the calculate density flag
	 * \param m_bCalculateDensity calculate density flag
	 */
	void setCalculateDensityFlag() {
		m_bCalculateDensity = false;
	};
	
	/** 
	 * sets the complete value of the volume data set or a specific
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
	 * \param fValue new value
	 */
	void fillVolume(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ, dt fValue) {
		
		int indexX;
		int indexY;
		int indexZ; 
		if (m_pData == NULL) {
			return;
		}
		if (iMaxX < m_iSizeX && iMaxY < m_iSizeY && iMaxZ < m_iSizeZ && iMinX >= 0 && iMinY >= 0 && iMinZ >= 0) {
  			for(indexZ=iMinZ;indexZ<=iMaxZ;++indexZ)
  				for(indexY=iMinY;indexY<=iMaxY;++indexY)
  					for(indexX=iMinX;indexX<=iMaxX;++indexX) {
							*(m_pData + pindex(indexZ, indexY, indexX)) = fValue;
  					}
			m_bCalculateDensity = true;
		}
	};
	
	void readVolume(char *vol_file) { 
		unsigned long nvox, count;
  		double dgridx, dgridy, dgridz, dwidth;
  		double phitest, dtemp;
  		FILE *fin;
  	 
  		fin = fopen(vol_file, "r");
  		if( fin == NULL ) {
  		  fprintf(stderr, "lib_vio> Error: Can't open file! %s  [e.c. 13010]\n", vol_file); 
  		  exit(13010);   
  		}

  		/* read header and print information */
  		fscanf(fin, "%le %le %le %le %d %d %d", &dwidth, &dgridx, &dgridy, &dgridz, &m_iSizeX, &m_iSizeY, &m_iSizeZ);
  		nvox = m_iSizeX * m_iSizeY * m_iSizeZ;
		cout << "Size X: " << m_iSizeX << " Size Y: " << m_iSizeY << " Size Z: " << m_iSizeY << endl;
  		/* allocate memory and read data */
  		printf ("lib_vio> Reading density data... \n");
		m_pData = (dt *) new dt[nvox];
  		if (m_pData == NULL) {
  			fprintf(stderr, "lib_vio> Error: Unable to satisfy memory allocation request [e.c. 13020]\n"); 
    			exit(13020);
  		}

  		for (count=0;count<nvox;count++) {
    			if (fscanf(fin,"%le", &dtemp) != 1) {
        			fprintf(stderr, "lib_vio> Error: file %s is too short (not compatible with header information) or data unreadable [e.c. 13030]\n", vol_file); 
        			exit(13030);
    			} else *(m_pData+count) = dtemp;
  		}
  		if (fscanf(fin,"%le", &phitest) != EOF) {
    			fprintf(stderr, "lib_vio> Error: file %s is too long, not compatible with header information [e.c. 13040]\n", vol_file); 
    			exit(13040);
  		}
  		fclose(fin);  
  		printf ("lib_vio> Volumetric data read from file %s\n", vol_file); 
	};
	/**
	 * sets the complete value of the volume data set
	 * \param fValue new value
	 */
	void fillVolume(dt fValue) {
		fillVolume(0, 0, 0, m_iSizeX - 1, m_iSizeY - 1, m_iSizeZ -1, fValue);
	};
	
	/**
	 * creates a new volume data set.  ATTENDION: This function will delete the old data set
	 * when called.
	 * \param iSizeX x size of the new volume data
	 * \param iSizeY y size of the new volume data
	 * \param iSizeZ z size of the new volume data
	 */
	void createVolume(int iSizeX, int iSizeY, int iSizeZ) {
		if (m_pData != NULL ) {
			delete[] m_pData;
		}
		m_pData = new dt[iSizeX * iSizeY * iSizeZ];
		m_iSizeX = iSizeX;
         m_iSizeY = iSizeY;
         m_iSizeZ = iSizeZ;
		m_bCalculateDensity = true;
	};

	
    /**
     * get the value at a position inside the volume
     * \param iX x coordinate
     * \param iY y coordinate
     * \param iZ z coordinate
     * \return value
     */
     virtual dt getValue(int iX, int iY, int iZ)
     {
         if (iX < m_iSizeX && iY < m_iSizeY && iZ < m_iSizeZ && iX >= 0 && iY >= 0 && iZ >= 0)
             return m_pData[pindex(iX,iY,iZ)];
         else
             return 0.0f;
     };

    /**
     * get the value at a position inside the volume
     * \param iCount counter
     * \return value
     */
     virtual dt getValue(int iCount) const
     {
         return m_pData[iCount];
     };
	
	/**
	 * get the density value
	 * \return value
	 */ 
	dt getDensity() {
		return m_fDensity;
	};

	/**
	 * set the density value
	 * \param density new density value
	 */ 
	void setDensity(dt density) {
		m_fDensity = density;
	};
	
	/**
	 * checks the pad parameters
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
	 */
	void padParametersCheck(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ) {

  		if (iMinX > 0) {
			iMinX = 0;
		}
  	  	if (iMaxX < m_iSizeX) {
			iMaxX = m_iSizeX - 1;
		}
		if (iMinY > 0) {
			iMinY = 0;
		}
		if (iMaxY < m_iSizeY) {
			iMaxY = m_iSizeY - 1;
		}
		if (iMinZ > 0) {
			iMinZ = 0;
		}
		if (iMaxZ < m_iSizeZ) {
			iMaxZ = m_iSizeZ - 1;
		}
	};
	
	/**
 	 * checks the crop parameters
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
    	 */
	void cropParametersCheck(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ) {

		if (iMinX < 0) {
			iMinX = 0;
		}
		if (iMinX >= m_iSizeX) {
			iMinX = m_iSizeX - 1;
  		}
  		if (iMaxX < iMinX) {
			iMaxX = iMinX;
		}
		if (iMaxX >= m_iSizeX) {
			iMaxX = m_iSizeX - 1;
  		}
		if (iMinY < 0) {
			iMinY = 0; 
		}
		if (iMinY >= m_iSizeY) {
			iMinY = m_iSizeY - 1;
  		}
		if (iMaxY < iMinY) {
			iMaxY = iMinY; 
		}
		if (iMaxY >= m_iSizeY) {
			iMaxY = m_iSizeY - 1;
  		}
		if (iMinZ < 0) {
			iMinZ = 0;
		}
		if (iMinZ >= m_iSizeZ) {
			iMinZ = m_iSizeZ - 1;
  		}
		if (iMaxZ < iMinZ) {
			iMaxZ = iMinZ; 
		}
		if (iMaxZ >= m_iSizeZ) {
			iMaxZ = m_iSizeZ - 1;
		}
	};

	/**
	 * crops a specified portion of the volume data 
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
	 */
	void crop(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ) {
		cropParametersCheck(iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ);
		createDensity(iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ, dt(0));
	};

	/**
	 * pads a specified portion of the volume data
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
	 * \param padval the value to pad the volume data
	 */
	void pad(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ, dt padval) {
		padParametersCheck(iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ);
		createDensity(iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ, padval);
	};
	
	/**
	 * creates a complete volume data set
	 * \param iMinX minimum x coordinate
	 * \param iMinY minimum y coordinate
	 * \param iMinZ minimum z coordinate
	 * \param iMaxX maximum x coordinate
	 * \param iMaxY maximum y coordinate
	 * \param iMaxZ maximum z coordinate
	 * \param padval the value to pad the volume data
	 */
	void createDensity(int iMinX, int iMinY, int iMinZ, int iMaxX, int iMaxY, int iMaxZ, dt padval) {
		
		dt *pphi;
		long int nvox;
		long int count;
		int indexX;
		int indexY;
		int indexZ; 

  		nvox = (iMaxX - iMinX + 1)*(iMaxY - iMinY + 1)*(iMaxZ - iMinZ + 1);
		pphi = (dt *) new dt[nvox];
  		if (pphi == NULL) {
    			fprintf(stderr, "volpad> Error: Unable to satisfy memory allocation request [e.c. 41010]\n");
    			exit(41010);
  		}
  		count = 0;
  		for(indexZ=iMinZ;indexZ<=iMaxZ;++indexZ)
  			for(indexY=iMinY;indexY<=iMaxY;++indexY)
  				for(indexX=iMinX;indexX<=iMaxX;++indexX) {
						if (indexZ < 0 || indexZ >= m_iSizeZ || indexY < 0 || indexY >= m_iSizeY || indexX < 0 || indexX >= m_iSizeX) 
							*(pphi+count) = padval;
						else
							*(pphi + count) = *(m_pData + pindex(indexZ, indexY, indexX));
						++count;
  				}
		delete[] m_pData;
		m_pData = pphi;
		m_iSizeX = iMaxX - iMinX + 1;
		m_iSizeY = iMaxY - iMinY + 1;
		m_iSizeZ = iMaxZ - iMinZ + 1;
		m_bCalculateDensity = true;
	};
	
	/**
	 * extract a targeted contiguous volume from a given volume data set
	 * \param iStartX x coordinate of the start voxel
	 * \param iStartY y coordinate of the start voxel
	 * \param iStartZ z coordinate of the start voxel
	 * \param cutoff density level
	 * \param shrinkFlag shrinking the size of the map about the found volume
	 */
	void floodFill(int iStartX, int iStartY, int iStartZ, dt cutoff, int shrinkFlag) {
		
		long int nvox = m_iSizeX*m_iSizeY*m_iSizeZ;
		long int count;
		double pcurrdist;
		double pmindist = 1e20;
		int iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ;
		int indexX;
		int indexY;
		int indexZ; 
		int xmindist = -1;
		int ymindist = -1;
		int zmindist = -1;
		int ncore = 0;
		int nngbr = 0;
		dt* pphi;
		
		cropParametersCheck(iStartX, iStartY,iStartZ, 0, 0, 0);
		pphi = (dt *) new dt[nvox];
		if (pphi == NULL) {
			fprintf(stderr, "floodfill> Error: Unable to satisfy memory allocation request [e.c. 44020]\n");
			exit(44020);
		}
		for (count=0;count<nvox;count++) *(pphi+count) = 0;

		for (indexZ=0;indexZ<m_iSizeZ;indexZ++)
			for (indexY=0;indexY<m_iSizeY;indexY++)
				for (indexX=0;indexX<m_iSizeX;indexX++)
					if (*(m_pData+pindex(indexZ, indexY, indexX))>=cutoff) {
						pcurrdist = (indexX-iStartX)*(indexX-iStartX)+(indexY-iStartY)*(indexY-iStartY)+(indexZ-iStartZ)*(indexZ-iStartZ);
						if (pcurrdist<pmindist) {
							pmindist = pcurrdist;
							xmindist = indexX;
							ymindist = indexY;
							zmindist = indexZ;
						}
					}

		if (xmindist == -1) {
			cout << "floodfill> No volume above density cutoff. Bye bye." << endl;
			exit(1);
		}

		if (xmindist != iStartX || ymindist != iStartY || zmindist != iStartZ) {
			iStartX = xmindist;
			iStartY = ymindist;
			iStartZ = zmindist;
		}

		cout << "floodfill> Filling contiguous volume..." << endl;
		fill(pphi,cutoff,iStartX,iStartY,iStartZ,m_iSizeX, m_iSizeY, m_iSizeZ);
		
		/* find neighbor elements of filled volume (these have values below cutoff) */
		for (indexZ=0;indexZ<m_iSizeZ;indexZ++)
			for (indexY=0;indexY<m_iSizeY;indexY++)
				for (indexX=0;indexX<m_iSizeX;indexX++)
					if (neighbor(pphi,indexX,indexY,indexZ, m_iSizeX, m_iSizeY, m_iSizeZ)) *(pphi+pindex(indexZ, indexY, indexX)) = 2;

		/* set outliers and neighbors below minimum density to the minimum density */
		for (count=0;count<nvox;count++) {
			if (*(pphi+count)==0) *(m_pData+count) = getDensity();
			else if (*(pphi+count)==2 && *(m_pData+count)<getDensity()) *(m_pData+count)=getDensity();
		}

		/* determine volumes of contiguous volume and remaining neighbors */
		for (count=0;count<nvox;count++) {
			if (*(pphi+count)==1) ++ncore;
			if (*(pphi+count)==2 && *(m_pData+count)>getDensity()) ++nngbr;
		}

		/* determine boundaries of floodfill volume and neighbors */

		iMinX = m_iSizeX-1;
		iMaxX = 0;
		iMinY = m_iSizeY-1;
		iMaxY = 0;
		iMinZ = m_iSizeZ-1;
		iMaxZ = 0;
		for (indexZ=0;indexZ<iStartZ;indexZ++)
			for (indexY=0;indexY<iStartY;indexY++)
				for (indexX=0;indexX<iStartX;indexX++)
					if (*(pphi+pindex(indexZ,indexY,indexX)) > 0) {
						if (indexX < iMinX) iMinX = indexX;
						if (indexX > iMaxX) iMaxX = indexX;
						if (indexY < iMinY) iMinY = indexY;
						if (indexY > iMaxY) iMaxY = indexY;
						if (indexZ < iMinZ) iMinZ = indexZ;
						if (indexZ > iMaxZ) iMaxZ = indexZ;
					}

		if ((iMaxX < iMinX) || (iMaxY < iMinY) || (iMaxZ < iMinZ)) {
			cout << "floodfill> Error: No volume found [e.c. 44030]" << endl;
			exit(44030);
		}

		if (shrinkFlag != 2) {
			iMinX = 0;
			iMaxX = iStartX-1;
			iMinY = 0;
			iMaxY = iStartY-1;
			iMinZ = 0;
			iMaxZ = iStartZ-1;
		}
		createDensity(iMinX, iMinY, iMinZ, iMaxX, iMaxY, iMaxZ, dt(0));
		delete[] pphi;
	};
	
protected:
	/**
	 * recursive floodfill function
	 * \param pphi pointer to the new data volume
	 * \param cutoff 
	 * \param iX x coordinate of the new volume data
	 * \param iY y coordinate of the new volume data
	 * \param iZ z coordinate of the new volume data
	 *
	void fill (dt *pphi, dt cutoff, int iX, int iY, int iZ) {
 
  		if ((*(m_pData+pindex(iZ,iY,iX)) >= cutoff)&&(*(pphi+pindex(iZ,iY,iX)) == 0)) {
    			*(pphi+pindex(iZ,iY,iX)) = 1;
    			if (iX > 0) {
      			fill(pphi,cutoff,iX-1,iY,iZ);
      			if (iY > 0) {
        				fill(pphi,cutoff,iX-1,iY-1,iZ);
        				if (iZ > 0) fill(pphi,cutoff,iX-1,iY-1,iZ-1);
        				if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX-1,iY-1,iZ+1);
      			}
      			if ((iY+1) < m_iSizeY) {
        				fill(pphi,cutoff,iX-1,iY+1,iZ);
        				if (iZ > 0) fill(pphi,cutoff,iX-1,iY+1,iZ-1);
        				if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX-1,iY+1,iZ+1);
      			}
      			if (iZ > 0) fill(pphi,cutoff,iX-1,iY,iZ-1);    
      			if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX-1,iY,iZ+1);
    			}
    			if ((iX+1) < m_iSizeX) {
      			fill(pphi,cutoff,iX+1,iY,iZ);
      			if (iY > 0) {
        				fill(pphi,cutoff,iX+1,iY-1,iZ);
        				if (iZ > 0) fill(pphi,cutoff,iX+1,iY-1,iZ-1);
        				if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX+1,iY-1,iZ+1);
      			}
      			if ((iY+1) < m_iSizeY) {
        				fill(pphi,cutoff,iX+1,iY+1,iZ);
        				if (iZ > 0) fill(pphi,cutoff,iX+1,iY+1,iZ-1);
        				if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX+1,iY+1,iZ+1);
      			}
      			if (iZ > 0) fill(pphi,cutoff,iX+1,iY,iZ-1);    
      			if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX+1,iY,iZ+1);
    			}
    			if (iY > 0) {
      			fill(pphi,cutoff,iX,iY-1,iZ);
      			if (iZ > 0) fill(pphi,cutoff,iX,iY-1,iZ-1);
      			if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX,iY-1,iZ+1);
    			}
    			if ((iY+1) < m_iSizeY) {
      			fill(pphi,cutoff,iX,iY+1,iZ);
      			if (iZ > 0) fill(pphi,cutoff,iX,iY+1,iZ-1);
      			if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX,iY+1,iZ+1);
    			}
    			if (iZ > 0) fill(pphi,cutoff,iX,iY,iZ-1);    
    			if ((iZ+1) < m_iSizeZ) fill(pphi,cutoff,iX,iY,iZ+1);
  		}
	};
	
	
	 * checks if index element is neighbor of floodfill element
	 * \param pphi pointer to the new data volume
	 * \param iX x coordinate of the new volume data
	 * \param iY y coordinate of the new volume data
	 * \param iZ z coordinate of the new volume data
	 * \return value
	 *
	int neighbor(dt *pphi, int iX, int iY, int iZ) {

 		if (*(pphi+pindex(iZ,iY,iX)) == 1) return 0;

  		if (iX > 0) {
    			if (*(pphi+pindex(iZ,iY,iX-1)) == 1) return 1;
    			if (iY > 0) {
      			if (*(pphi+pindex(iZ,iY-1,iX-1)) == 1) return 1;
      			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY-1,iX-1)) == 1) return 1;
      			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY-1,iX-1)) == 1) return 1;
    			}
    			if ((iY+1) < m_iSizeY) {
      			if (*(pphi+pindex(iZ,iY+1,iX-1)) == 1) return 1;
      			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY+1,iX-1)) == 1) return 1;
      			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY+1,iX-1)) == 1) return 1;
    			}
    			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY,iX-1)) == 1) return 1;
    			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY,iX-1)) == 1) return 1;
  		}
  		if ((iX+1) < m_iSizeX) {
    			if (*(pphi+pindex(iZ,iY,iX+1)) == 1) return 1;
    			if (iY > 0) {
     	 		if (*(pphi+pindex(iZ,iY-1,iX+1)) == 1) return 1;
     	 		if (iZ > 0) if (*(pphi+pindex(iZ-1,iY-1,iX+1)) == 1) return 1;
     	 		if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY-1,iX+1)) == 1) return 1;
    			}
    			if ((iY+1) < m_iSizeY) {
     	 		if (*(pphi+pindex(iZ,iY+1,iX+1)) == 1) return 1;
     	 		if (iZ > 0) if (*(pphi+pindex(iZ-1,iY+1,iX+1)) == 1) return 1;
     	 		if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY+1,iX+1)) == 1) return 1;
    			}
    			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY,iX+1)) == 1) return 1;
    			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY,iX+1)) == 1) return 1;
  		}
  		if (iY > 0) {
    			if (*(pphi+pindex(iZ,iY-1,iX)) == 1) return 1;
    			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY-1,iX)) == 1) return 1;
    			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY-1,iX)) == 1) return 1;
  		}
  		if ((iY+1) < m_iSizeY) {
    			if (*(pphi+pindex(iZ,iY+1,iX)) == 1) return 1;
    			if (iZ > 0) if (*(pphi+pindex(iZ-1,iY+1,iX)) == 1) return 1;
    			if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY+1,iX)) == 1) return 1;
  		}
  		if (iZ > 0) if (*(pphi+pindex(iZ-1,iY,iX)) == 1) return 1;
  		if ((iZ+1) < m_iSizeZ) if (*(pphi+pindex(iZ+1,iY,iX)) == 1) return 1;
  		return 0;
	}; */
	
	void fill (dt *iphi, double cutoff, int l, int m, int n, int extx, int exty, int extz) {

	/* recursive floodfill function */
 
  	if ((*(m_pData+gidz_general(n,m,l,exty,extx)) >= cutoff)&&(*(iphi+gidz_general(n,m,l,exty,extx)) == 0)) {
    		*(iphi+gidz_general(n,m,l,exty,extx)) = 1;
    		if (l > 0) {
      		fill(iphi,cutoff,l-1,m,n,extx,exty,extz);
      		if (m > 0) {
        			fill(iphi,cutoff,l-1,m-1,n,extx,exty,extz);
        			if (n > 0) fill(iphi,cutoff,l-1,m-1,n-1,extx,exty,extz);
        			if ((n+1) < extz) fill(iphi,cutoff,l-1,m-1,n+1,extx,exty,extz);
      		}
      		if ((m+1) < exty) {
        			fill(iphi,cutoff,l-1,m+1,n,extx,exty,extz);
        			if (n > 0) fill(iphi,cutoff,l-1,m+1,n-1,extx,exty,extz);
        			if ((n+1) < extz) fill(iphi,cutoff,l-1,m+1,n+1,extx,exty,extz);
      		}
      		if (n > 0) fill(iphi,cutoff,l-1,m,n-1,extx,exty,extz);    
      		if ((n+1) < extz) fill(iphi,cutoff,l-1,m,n+1,extx,exty,extz);
    		}
    		if ((l+1) < extx) {
      		fill(iphi,cutoff,l+1,m,n,extx,exty,extz);
      		if (m > 0) {
        			fill(iphi,cutoff,l+1,m-1,n,extx,exty,extz);
        			if (n > 0) fill(iphi,cutoff,l+1,m-1,n-1,extx,exty,extz);
        			if ((n+1) < extz) fill(iphi,cutoff,l+1,m-1,n+1,extx,exty,extz);
      		}
      		if ((m+1) < exty) {
        			fill(iphi,cutoff,l+1,m+1,n,extx,exty,extz);
        			if (n > 0) fill(iphi,cutoff,l+1,m+1,n-1,extx,exty,extz);
        			if ((n+1) < extz) fill(iphi,cutoff,l+1,m+1,n+1,extx,exty,extz);
      		}
      		if (n > 0) fill(iphi,cutoff,l+1,m,n-1,extx,exty,extz);    
      		if ((n+1) < extz) fill(iphi,cutoff,l+1,m,n+1,extx,exty,extz);
    		}
    		if (m > 0) {
      		fill(iphi,cutoff,l,m-1,n,extx,exty,extz);
      		if (n > 0) fill(iphi,cutoff,l,m-1,n-1,extx,exty,extz);
      		if ((n+1) < extz) fill(iphi,cutoff,l,m-1,n+1,extx,exty,extz);
    		}
    		if ((m+1) < exty) {
      		fill(iphi,cutoff,l,m+1,n,extx,exty,extz);
      		if (n > 0) fill(iphi,cutoff,l,m+1,n-1,extx,exty,extz);
      		if ((n+1) < extz) fill(iphi,cutoff,l,m+1,n+1,extx,exty,extz);
    		}
    		if (n > 0) fill(iphi,cutoff,l,m,n-1,extx,exty,extz);    
    		if ((n+1) < extz) fill(iphi,cutoff,l,m,n+1,extx,exty,extz);
  	}
  	return;
};

	int neighbor(dt *iphi, int l, int m, int n, int extx, int exty, int extz) {

	/* checks if index element is neighbor of floodfill element */

 	if (*(iphi+gidz_general(n,m,l,exty,extx)) == 1) return 0;

  	if (l > 0) {
    		if (*(iphi+gidz_general(n,m,l-1,exty,extx)) == 1) return 1;
    		if (m > 0) {
      		if (*(iphi+gidz_general(n,m-1,l-1,exty,extx)) == 1) return 1;
      		if (n > 0) if (*(iphi+gidz_general(n-1,m-1,l-1,exty,extx)) == 1) return 1;
      		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m-1,l-1,exty,extx)) == 1) return 1;
    		}
    		if ((m+1) < exty) {
      		if (*(iphi+gidz_general(n,m+1,l-1,exty,extx)) == 1) return 1;
      		if (n > 0) if (*(iphi+gidz_general(n-1,m+1,l-1,exty,extx)) == 1) return 1;
      		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m+1,l-1,exty,extx)) == 1) return 1;
    		}
    		if (n > 0) if (*(iphi+gidz_general(n-1,m,l-1,exty,extx)) == 1) return 1;
    		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m,l-1,exty,extx)) == 1) return 1;
  	}
  	if ((l+1) < extx) {
    		if (*(iphi+gidz_general(n,m,l+1,exty,extx)) == 1) return 1;
    		if (m > 0) {
      		if (*(iphi+gidz_general(n,m-1,l+1,exty,extx)) == 1) return 1;
      		if (n > 0) if (*(iphi+gidz_general(n-1,m-1,l+1,exty,extx)) == 1) return 1;
      		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m-1,l+1,exty,extx)) == 1) return 1;
    		}
    		if ((m+1) < exty) {
      		if (*(iphi+gidz_general(n,m+1,l+1,exty,extx)) == 1) return 1;
      		if (n > 0) if (*(iphi+gidz_general(n-1,m+1,l+1,exty,extx)) == 1) return 1;
      		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m+1,l+1,exty,extx)) == 1) return 1;
    		}
    		if (n > 0) if (*(iphi+gidz_general(n-1,m,l+1,exty,extx)) == 1) return 1;
    		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m,l+1,exty,extx)) == 1) return 1;
  	}
  	if (m > 0) {
    		if (*(iphi+gidz_general(n,m-1,l,exty,extx)) == 1) return 1;
    		if (n > 0) if (*(iphi+gidz_general(n-1,m-1,l,exty,extx)) == 1) return 1;
    		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m-1,l,exty,extx)) == 1) return 1;
  	}
  	if ((m+1) < exty) {
    		if (*(iphi+gidz_general(n,m+1,l,exty,extx)) == 1) return 1;
    		if (n > 0) if (*(iphi+gidz_general(n-1,m+1,l,exty,extx)) == 1) return 1;
    		if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m+1,l,exty,extx)) == 1) return 1;
  	}
  	if (n > 0) if (*(iphi+gidz_general(n-1,m,l,exty,extx)) == 1) return 1;
  	if ((n+1) < extz) if (*(iphi+gidz_general(n+1,m,l,exty,extx)) == 1) return 1;
  	return 0;
};

public:
    /**
     * calculate the minimum density
     */
     void calcMinMaxDensity()
     {
         long int count;
         long int nvox = m_iSizeX*m_iSizeY*m_iSizeZ;

         /* compute range of density and set cutoff */
         m_fMaxDensity = getValue(0,0,0);
         m_fMinDensity = getValue(0,0,0);
         for (count=0;count<nvox;count++) {
             if (m_fMaxDensity < getValue(count)) m_fMaxDensity = getValue(count);
             if (m_fMinDensity > getValue(count)) m_fMinDensity = getValue(count);
         }
		setCalculateDensityFlag();
     };


    /**
     * get the minimum density.
     * \return the minimum density
     */
     dt getMinDensity()
     {
         if(getCalculateDensityFlag()) {
			calcMinMaxDensity();
		}
		return m_fMinDensity;
     };

    /**
     * get the maximum density.
     *  \return the maximum density
     */
     dt getMaxDensity()
     {
		if(getCalculateDensityFlag()) {
			calcMinMaxDensity();
		}
         return m_fMaxDensity;
     };
	 
protected:

    /**
     * returns index of cross section point
     */
    unsigned long pindex (int i, int j, int k) const
    {
		return i+j*m_iSizeX+k*m_iSizeX*m_iSizeY;
    };
	
	unsigned long gidz_general (int i, int j, int k, int iSizeX, int iSizeY) const
    {
		return i+j*iSizeX+k*iSizeX*iSizeY;
    };

};

#endif
